
/****************************************************************************
 National Instruments Corporation.

 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 ****************************************************************************/

#ifndef _SI514_REGMAP_H_
#define _SI514_REGMAP_H_

/****************************************************************************
 * Si514 Regmap *
 ****************************************************************************/

/* Si514 Registers Offset */
#define SI514_REG_OFFSET_M_FRAC1  5
#define SI514_REG_OFFSET_M_FFRAC2  6
#define SI514_REG_OFFSET_M_FRAC3  7
#define SI514_REG_OFFSET_M_INT_FRACT 8
#define SI514_REG_OFFSET_M_INT  9
#define SI514_REG_OFFSET_HS_DIV  10
#define SI514_REG_OFFSET_LS_HS_DIV 11

/* Register Masks */
#define SI514_RESET_RST  BIT(7)
#define SI514_CONTROL_FCAL BIT(0)
#define SI514_CONTROL_OE BIT(2)

/* Default adjustment range comes from the 30000pbb max
** stability of the 514 part itself (it needs to be at least
** double that so that two 514-based devices at opposite
** ranges will be able to sync to each other) plus a 2ppm
** extra tuning range for the servo. */
#define SI514_DEFAULT_ADJ_PPB 62000

/* Fixed-frequency Crystal, 29-bit resolution */
static const unsigned int si514_FXO = 31980000;
/* 536870912 dec,  Scaling factor for the 29-bit */
static const unsigned int si514_29bit_val = 0x20000000;
static const unsigned int si514_Ppb_divisor = 1000000000;
static const int fraction_scale = 1000;

/* Multiplier/divider settings */
struct tSi514_muldiv {
	__u32 m_frac; /* 29-bit Fractional part of multiplier M */
	__u8 m_int; /* Integer part of multiplier M, 65..78 */
	__u8 ls_div_bits; /* 2nd divider, as 2^x */
	__u16 hs_div; /* 1st divider, must be even and 10<=x<=1022 */
};

/*
 * Default values for 100Mhz operation (from powerup state of 100Mhz
 * factory part)
 */
static const struct tSi514_muldiv k100MhzReferenceHWSettings = {
	.m_frac = 425736282,
	.m_int = 68,
	.ls_div_bits = 0,
	.hs_div = 22
};

/* Default values for 25Mhz operation (from Si514 datasheet table) */
static const struct tSi514_muldiv k25MhzReferenceHWSettings = {
	.m_frac = 357578187,
	.m_int = 65,
	.ls_div_bits = 0,
	.hs_div = 84
};

/** Device State (client specific and additional data) **/
struct si514_data {
	struct i2c_client *i2c_client;
	struct mutex lock;
	struct tSi514_muldiv current_hw_settings;
	const struct tSi514_muldiv *reference_hw_settings;
};

#endif /* _S514_REGMAP_H_ */